import { TypedUseSelectorHook, useDispatch, useSelector } from 'react-redux';
import { GraphQueryResult, selectGraphQueryResult, selectGraphQueryResultMetaData } from './graphQueryResultSlice';
import {
  schemaGraph,
  selectSchemaLayout,
  schemaSettingsState,
  SchemaSettings,
  schemaStatsState,
  schemaInferenceState,
} from './schemaSlice';
import type { RootState, AppDispatch } from './store';
import { ConfigStateI, configState } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import {
  QueryBuilderSettings,
  QueryBuilderState,
  queryBuilderAttributesShown,
  queryBuilderSettingsState,
  queryBuilderState,
  selectQuerybuilderGraph,
  selectQuerybuilderHash,
} from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import { SessionCacheI, sessionCacheState } from './sessionSlice';
import { UseIsAuthorizedState, authState } from './authSlice';
import { visualizationState, VisState, visualizationActive } from './visualizationSlice';
import { ML, allMLEnabled, selectML } from './mlSlice';
import {
  searchResultState,
  searchResultData,
  searchResultSchema,
  searchResultQB,
  recentSearches,
  SearchCategoryMapI,
  CategoryDataI,
} from './searchResultSlice';
import { AllLayoutAlgorithms } from '../../graph-layout';
import { QueryGraphEdgeHandle, QueryMultiGraph } from '../../querybuilder';
import { SchemaGraph, SchemaGraphInference, SchemaGraphStats } from '../../schema';
import { GraphMetadata } from '../statistics';
import { SelectionStateI, FocusStateI, focusState, selectionState } from './interactionSlice';
import { VisualizationSettingsType } from '../../vis/common';

// Use throughout your app instead of plain `useDispatch` and `useSelector`
export const useAppDispatch: () => AppDispatch = useDispatch;
// export const useAppDispatch = () => useDispatch<AppDispatch>();
export const useAppSelector: TypedUseSelectorHook<RootState> = useSelector;

/** Gives the graphQueryResult from the store */
export const useGraphQueryResult: () => GraphQueryResult = () => useAppSelector(selectGraphQueryResult);
export const useGraphQueryResultMeta: () => GraphMetadata | undefined = () => useAppSelector(selectGraphQueryResultMetaData);

// Gives the schema
export const useSchemaGraph: () => SchemaGraph = () => useAppSelector(schemaGraph);
export const useSchemaSettings: () => SchemaSettings = () => useAppSelector(schemaSettingsState);
export const useSchemaStats: () => SchemaGraphStats = () => useAppSelector(schemaStatsState);
export const useSchemaInference: () => SchemaGraphInference = () => useAppSelector(schemaInferenceState);
export const useSchemaLayout: () => AllLayoutAlgorithms = () => useAppSelector(selectSchemaLayout);

// Querybuilder Slices
export const useQuerybuilderGraph: () => QueryMultiGraph = () => useAppSelector(selectQuerybuilderGraph);
export const useQuerybuilderHash: () => string = () => useAppSelector(selectQuerybuilderHash);
export const useQuerybuilderSettings: () => QueryBuilderSettings = () => useAppSelector(queryBuilderSettingsState);
export const useQuerybuilder: () => QueryBuilderState = () => useAppSelector(queryBuilderState);
export const useQuerybuilderAttributesShown: () => QueryGraphEdgeHandle[] = () => useAppSelector(queryBuilderAttributesShown);

// Overall Configuration of the app
export const useConfig: () => ConfigStateI = () => useAppSelector(configState);
export const useSessionCache: () => SessionCacheI = () => useAppSelector(sessionCacheState);
export const useAuthorizationCache: () => UseIsAuthorizedState = () => useAppSelector(authState);

// Machine Learning Slices
export const useML: () => ML = () => useAppSelector(selectML);
export const useMLEnabledHash: () => string = () => useAppSelector(allMLEnabled);

// Search Result Slices
export const useSearchResult: () => SearchCategoryMapI = () => useAppSelector(searchResultState);
export const useSearchResultData: () => CategoryDataI = () => useAppSelector(searchResultData);
export const useSearchResultSchema: () => any[] = () => useAppSelector(searchResultSchema);
export const useSearchResultQB: () => CategoryDataI = () => useAppSelector(searchResultQB);
export const useRecentSearches: () => string[] = () => useAppSelector(recentSearches);

// Visualization Slices
export const useVisualization: () => VisState = () => useAppSelector(visualizationState);
export const useActiveVisualization: () => VisualizationSettingsType | undefined = () => useAppSelector(visualizationActive);

// Interaction Slices
export const useSelection: () => SelectionStateI | undefined = () => useAppSelector(selectionState);
export const useFocus: () => FocusStateI | undefined = () => useAppSelector(focusState);
