import React from 'react';
import { Icon } from '@graphpolaris/shared/lib/components/icon';
import { Tooltip, TooltipTrigger, TooltipContent, TooltipProvider } from '@graphpolaris/shared/lib/components/tooltip';

const formatNumber = (number: number) => {
  return number.toLocaleString('de-DE');
};

export type SchemaPopUpProps = {
  data: Record<string, any>;
  connections?: { to: string; from: string };
  numberOfElements: any;
};

export const SchemaPopUp: React.FC<SchemaPopUpProps> = ({ data, numberOfElements, connections }) => {
  return (
    <>
      <div className="">
        {numberOfElements && (
          <div className="border-b border-sec-200">
            <div className="flex flex-row gap-1 items-center justify-between px-3 py-1">
              <Icon component="icon-[ic--baseline-numbers]" size={24} />
              <span className="ml-auto text-right">{formatNumber(numberOfElements)}</span>
            </div>
          </div>
        )}
        {connections && (
          <div className="border-b border-sec-200 px-3 py-1">
            <div className="flex flex-row gap-3 items-center justify-between">
              <span className="font-semibold">From</span>
              <span className="ml-auto text-right">{connections.from}</span>
            </div>
            <div className="flex flex-row gap-1 items-center justify-between">
              <span className="font-semibold">To</span>
              <span className="ml-auto text-right">{connections.to}</span>
            </div>
          </div>
        )}
        <TooltipProvider delayDuration={300}>
          <div className="px-3 py-1">
            {Object.keys(data).length === 0 ? (
              <div className="flex justify-center items-center h-full ">
                <span>No attributes</span>
              </div>
            ) : (
              Object.entries(data).map(([k, v]) => (
                <Tooltip key={k}>
                  <div className="flex flex-row gap-1 items-center min-h-6">
                    <span className={`font-semibold truncate w-[90%]`}>{k}</span>
                    <TooltipTrigger asChild>
                      <span className="ml-auto text-right truncate grow-1 flex items-center">
                        <Icon
                          className="ml-auto text-right flex-shrink-0"
                          component={
                            v === 'int' || v === 'float' ? (
                              <Icon component="icon-[carbon--string-integer]" size={24} color={'hsl(var(--clr-sec--500))'} />
                            ) : v === 'string' ? (
                              <Icon component="icon-[carbon--string-text]" size={24} color={'hsl(var(--clr-sec--500))'} />
                            ) : v === 'boolean' || v === 'bool' ? (
                              <Icon component="icon-[carbon--boolean]" size={24} color={'hsl(var(--clr-sec--500))'} />
                            ) : v === 'date' || v === 'time' || v === 'duration' || v === 'datetime' ? (
                              <Icon component="icon-[carbon--calendar]" size={24} color={'hsl(var(--clr-sec--500))'} />
                            ) : v === 'undefined' ? (
                              <Icon component="icon-[carbon--undefined]" size={24} color={'hsl(var(--clr-sec--500))'} />
                            ) : (
                              <Icon component="icon-[carbon--undefined]" size={24} color={'hsl(var(--clr-sec--500))'} />
                            )
                          }
                          color="hsl(var(--clr-sec--400))"
                          size={24}
                        />
                      </span>
                    </TooltipTrigger>
                    <TooltipContent side="right">
                      <div className="max-w-[18rem] break-all line-clamp-6 mx-1">
                        {v !== undefined && (typeof v !== 'object' || Array.isArray(v)) && v != '' ? v : 'noData'}
                      </div>
                    </TooltipContent>
                  </div>
                </Tooltip>
              ))
            )}
          </div>
        </TooltipProvider>
      </div>
    </>
  );
};
